/*
 ATtiny GPS LongLat Logger
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 9/10/2013
 Last Modified: 9/10/2013
 
 Receive GPS NMEA data at 9600bps via USART on the ATtiny2313A, parse some of the data and write the 
 latitude and longitude to an external I2C EEPROM.
 
 */
 
// ATtiny2313A Pin Map
//
//              +-\/-+
//  Reset PA2  1|o   |20  VCC
//  RX    PD0  2|    |19  PB7
//  TX    PD1  3|    |18  PB6
//        PA1  4|    |17  PB5
//        PA0  5|    |16  PB4 SDA for EEPROM
//        PD2  6|    |15  PB3 SCL for EEPROM
//        PD3  7|    |14  PB2
//        PD4  8|    |13  PB1
//        PD5  9|    |12  PB0
//        GND 10|    |11  PD6
//              +----+

#define F_CPU 1000000 // 1 MHz

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <string.h>
#include <math.h>
#include <util/delay.h>
#include "setup.c"

#define EEPROM_ADDR 0x50 // I2C EEPROM address
#define EEPROM_END_ADDR 8192 // For 64Kbit EEPROM
#define GPS_LONG_LAT_SIZE 22 // How many bytes are the latitude and longitude are combined

uint16_t dataCount = 0; // Current EEPROM location
uint8_t gpsLatitude[15];
uint8_t gpsLongitude[15];
uint8_t tempBuffer[15];
uint8_t gpsLogdelay = 1; // Log every 2nd GPS long/lat that we receive to the EEPROM (every 2 seconds with a 1 PPS GPS)
uint8_t gpsLogdelaycounter = 0; 

// Receive USART data
uint8_t USART_Receive( void ) {
	while ( !(UCSRA & (1<<RXC)) ); // Wait for data to be received
	return UDR; // Get and return received data from buffer
}

// Transmit USART data
void USART_Transmit(unsigned char data) {
	while ( !( UCSRA & (1<<UDRE)) ); // Wait for empty transmit buffer
	UDR = data;
}

// Read USART until we reach a comma
byte read_usart_to_comma(void) {
	int x = 0;
	tempBuffer[x] = USART_Receive();
	
	while (tempBuffer[x] != ',') {
		x++;
		tempBuffer[x] = USART_Receive();
	}
	
	return x;
}

int main(void) {
	setup();
	
	while (1) {
		uint8_t rxData = USART_Receive();
		
		// Start of frame
		if (rxData == '$') {
			// Read next chars
			read_usart_to_comma();
			
			// Check if GPRMC
			if (memcmp(tempBuffer, "GPRMC", 5) == 0) {
				read_usart_to_comma(); // Timestamp (ignore)
				read_usart_to_comma(); // Validity
				
				// If valid GPS data
				if (tempBuffer[0] == 'A') {
					if (gpsLogdelaycounter >= gpsLogdelay && dataCount < (EEPROM_END_ADDR - GPS_LONG_LAT_SIZE)) {
						uint8_t bytesReadLatitude = read_usart_to_comma(); // Latitude
						memcpy(gpsLatitude, tempBuffer, bytesReadLatitude);
						
						read_usart_to_comma(); // N or S (ignore because I know what it's going to be, you should check this)
						
						uint8_t bytesReadLongitude = read_usart_to_comma(); // Longitude
						memcpy(gpsLongitude, tempBuffer, bytesReadLongitude);
						
						// Write to I2C EEPROM
						for (int z = 0; z < bytesReadLatitude; z++) {
							//USART_Transmit(gpsLatitude[z]);
							soft_i2c_eeprom_write_byte(EEPROM_ADDR, dataCount, gpsLatitude[z]);
							dataCount++;
						}
						for (int z = 0; z < bytesReadLongitude; z++) {
							//USART_Transmit(gpsLongitude[z]);
							soft_i2c_eeprom_write_byte(EEPROM_ADDR, dataCount, gpsLongitude[z]);
							dataCount++;
						}
						
						gpsLogdelaycounter = 0;
					}
					else {
						gpsLogdelaycounter++;
					}
				}
			}
		}
	}
}
